/*
 * ====================================================================
 * 
 * The ObjectStyle Group Software License, Version 1.0
 * 
 * Copyright (c) 2006 The ObjectStyle Group and individual authors of the
 * software. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if any, must
 * include the following acknowlegement: "This product includes software
 * developed by the ObjectStyle Group (http://objectstyle.org/)." Alternately,
 * this acknowlegement may appear in the software itself, if and wherever such
 * third-party acknowlegements normally appear.
 * 
 * 4. The names "ObjectStyle Group" and "Cayenne" must not be used to endorse or
 * promote products derived from this software without prior written permission.
 * For written permission, please contact andrus@objectstyle.org.
 * 
 * 5. Products derived from this software may not be called "ObjectStyle" nor
 * may "ObjectStyle" appear in their names without prior written permission of
 * the ObjectStyle Group.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * OBJECTSTYLE GROUP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by many individuals on
 * behalf of the ObjectStyle Group. For more information on the ObjectStyle
 * Group, please see <http://objectstyle.org/>.
 *  
 */
package org.objectstyle.wolips.eomodeler.utils;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Display;
import org.objectstyle.wolips.baseforplugins.util.Throttle;

public class SelectionThrottle implements Runnable {
	private String _name;
	
	private Throttle _throttle;

	private IThrottledSelectionHandler _selectionHandler;

	private List _addedObjects;

	private List _removedObjects;

	public SelectionThrottle(String name, IThrottledSelectionHandler selectionHandler) {
		_name = name;
		_selectionHandler = selectionHandler;
		_addedObjects = new LinkedList();
		_removedObjects = new LinkedList();
		_throttle = new Throttle(_name, 150, this);
	}

	public void start() {
		_throttle.start();
	}

	public void stop() {
		_throttle.stop();
	}

	public void run() {
		final List addedObjects;
		synchronized (_addedObjects) {
			addedObjects = _addedObjects;
			_addedObjects = new LinkedList();
		}
		// final List removedObjects;
		synchronized (_removedObjects) {
			// removedObjects = _removedObjects;
			_removedObjects = new LinkedList();
		}
		if (!addedObjects.isEmpty()) {
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					_selectionHandler.changeSelection(new StructuredSelection(addedObjects.get(0)));
				}
			});
		}
	}

	public void objectsAdded(Collection addedObjects) {
		if (!addedObjects.isEmpty()) {
			synchronized (_addedObjects) {
				_addedObjects.addAll(addedObjects);
			}
			_throttle.ping();
		}
	}

	public void objectsRemoved(Collection removedObjects) {
		if (!removedObjects.isEmpty()) {
			synchronized (_removedObjects) {
				_removedObjects.addAll(removedObjects);
			}
			_throttle.ping();
		}
	}

	public static class TableViewerSelectionHandler implements IThrottledSelectionHandler {
		private TableViewer _tableViewer;

		public TableViewerSelectionHandler(TableViewer tableViewer) {
			_tableViewer = tableViewer;
		}

		public void changeSelection(ISelection selection) {
			_tableViewer.setSelection(selection);
			_tableViewer.getTable().forceFocus();
		}
	}

	public static interface IThrottledSelectionHandler {
		public void changeSelection(ISelection selection);
	}
}
