/*
 * ====================================================================
 * 
 * The ObjectStyle Group Software License, Version 1.0
 * 
 * Copyright (c) 2005 The ObjectStyle Group and individual authors of the
 * software. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 1.
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. 2. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. 3. The end-user documentation
 * included with the redistribution, if any, must include the following
 * acknowlegement: "This product includes software developed by the ObjectStyle
 * Group (http://objectstyle.org/)." Alternately, this acknowlegement may
 * appear in the software itself, if and wherever such third-party
 * acknowlegements normally appear. 4. The names "ObjectStyle Group" and
 * "Cayenne" must not be used to endorse or promote products derived from this
 * software without prior written permission. For written permission, please
 * contact andrus@objectstyle.org. 5. Products derived from this software may
 * not be called "ObjectStyle" nor may "ObjectStyle" appear in their names
 * without prior written permission of the ObjectStyle Group.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * OBJECTSTYLE GROUP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by many individuals
 * on behalf of the ObjectStyle Group. For more information on the ObjectStyle
 * Group, please see <http://objectstyle.org/> .
 *  
 */
package org.objectstyle.wolips.wodclipse.core.parser;

import org.eclipse.jface.text.rules.ICharacterScanner;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.Token;

/**
 * @author mschrag
 */
public class MultilineCommentRule implements ICommentRule {
	private static final int COMMENT_STATE_NONE = 0;

	private static final int COMMENT_STATE_SLASH = 1;

	private static final int COMMENT_STATE_SLASHSTAR = 2;

	private static final int COMMENT_STATE_SLASHSTARSTAR = 3;

	private static final int COMMENT_STATE_SLASHSTARSTARSLASH = 4;

	private IToken myToken;

	public MultilineCommentRule(IToken _token) {
		myToken = _token;
	}

	public IToken getSuccessToken() {
		return myToken;
	}

	public IToken evaluate(ICharacterScanner _scanner) {
		return evaluate(_scanner, false);
	}

	public IToken evaluate(ICharacterScanner _scanner, boolean _resume) {
		//int startColumn = _scanner.getColumn();
		IToken token = Token.UNDEFINED;
		int ch = 0;
		int unreadCount = 0;
		int commentState = MultilineCommentRule.COMMENT_STATE_NONE;
		boolean done = false;
		while (!done && (ch = _scanner.read()) != ICharacterScanner.EOF) {
			if (commentState == MultilineCommentRule.COMMENT_STATE_NONE) {
				unreadCount++;
				if (ch == '/') {
					commentState = MultilineCommentRule.COMMENT_STATE_SLASH;
				} else {
					done = true;
				}
			} else if (commentState == MultilineCommentRule.COMMENT_STATE_SLASH) {
				if (ch == '*') {
					commentState = MultilineCommentRule.COMMENT_STATE_SLASHSTAR;
					token = myToken;
					unreadCount = 0;
				} else {
					unreadCount++;
					commentState = MultilineCommentRule.COMMENT_STATE_NONE;
					done = true;
				}
			} else if (commentState == MultilineCommentRule.COMMENT_STATE_SLASHSTAR) {
				if (ch == '*') {
					commentState = MultilineCommentRule.COMMENT_STATE_SLASHSTARSTAR;
				}
			} else if (commentState == MultilineCommentRule.COMMENT_STATE_SLASHSTARSTAR) {
				if (ch == '/') {
					commentState = MultilineCommentRule.COMMENT_STATE_SLASHSTARSTARSLASH;
					done = true;
				} else {
					commentState = MultilineCommentRule.COMMENT_STATE_SLASHSTARSTAR;
				}
			}
		}

		if (ch == ICharacterScanner.EOF) {
			unreadCount++;
		}
		for (int i = 0; i < unreadCount; i++) {
			_scanner.unread();
		}

		return token;
	}
}
